@@ -47,6 +47,7 @@ signed int min_batt=-1;
 int use_simplehal = 0;
 #endif
 int use_acpi=0;
+int force_hal=0;
 int require_unused_and_battery=0;	/* --and or -A option */
 double max_loadavg = 0;
 int use_utmp=0;
@@ -58,7 +59,7 @@ char netdevrx[MAX_NET][44];
 int debug=0;
 
 void usage () {
-	fprintf(stderr, "Usage: sleepd [-s command] [-d command] [-u n] [-U n] [-I] [-i n] [-E] [-e filename] [-a] [-l n] [-w] [-n] [-v] [-c n] [-b n] [-A] [-N [dev] [-t n] [-r n]]\n");
+	fprintf(stderr, "Usage: sleepd [-s command] [-d command] [-u n] [-U n] [-I] [-i n] [-E] [-e filename] [-a] [-l n] [-w] [-n] [-v] [-c n] [-b n] [-A] [-H] [-N [dev] [-t n] [-r n]]\n");
 }
 
 void parse_command_line (int argc, char **argv) {
@@ -84,6 +85,7 @@ void parse_command_line (int argc, char **argv) {
 		{"netdev", 2, NULL, 'N'},
 		{"rx-min", 1, NULL, 'r'},
 		{"tx-min", 1, NULL, 't'},
+		{"force-hal", 0, NULL, 'H'},
 		{0, 0, 0, 0}
 	};
 	int force_autoprobe=0;
@@ -98,7 +100,7 @@ void parse_command_line (int argc, char **argv) {
 	char rx_statfile[44];
 
 	while (c != -1) {
-		c=getopt_long(argc,argv, "s:d:nvu:U:l:wIi:Ee:hac:b:AN::r:t:", long_options, NULL);
+		c=getopt_long(argc,argv, "s:d:nvu:U:l:wIi:Ee:hac:b:AN::r:t:H", long_options, NULL);
 		switch (c) {
 			case 's':
 				sleep_command=strdup(optarg);
@@ -124,6 +126,9 @@ void parse_command_line (int argc, char **argv) {
 			case 'w':
 				use_utmp=1;
 				break;
+			case 'H':
+				force_hal=1;
+				break;
 			case 'i':
 				i = atoi(optarg);
 				if ((i < 0) || (i >= MAX_IRQS)) {
@@ -412,14 +417,19 @@ void main_loop (void) {
 	int prev_ac_line_status=0;
 	time_t nowtime, oldtime=0;
 	apm_info ai;
-	pthread_t emthread;
-	eventData.timeout = sleep_time;
 	double loadavg[1];
 
+	if (use_events) {
+		pthread_t emthread;
+		pthread_create(&emthread, NULL, eventMonitor, NULL);
+	}
+
 	while (1) {
 		activity=0;
 		if (use_events) {
-			pthread_create(&emthread, NULL, eventMonitor, NULL);
+			pthread_mutex_lock(&condition_mutex);
+			pthread_cond_signal(&condition_cond);
+			pthread_mutex_unlock(&condition_mutex);
 		}
 
 		if (use_acpi) {
@@ -486,16 +496,16 @@ void main_loop (void) {
 		}
 		prev_ac_line_status=ai.ac_line_status;
 
+		sleep(sleep_time);
+
 		if (use_events) {
-			pthread_join(emthread, NULL);
+			pthread_mutex_lock(&activity_mutex);
 			if (eventData.emactivity == 1) {
 				if (debug)
 					printf("sleepd: activity: keyboard/mouse events\n");
 				activity=1;
 			}
-		}
-		else {
-			sleep(sleep_time);
+			pthread_mutex_unlock(&activity_mutex);
 		}
 
 		if (activity) {
@@ -595,7 +605,7 @@ int main (int argc, char **argv) {
 		}
 	}
 	
-	if (apm_exists() != 0) {
+	if (force_hal || apm_exists() != 0) {
 		if (! sleep_command)
 			sleep_command=acpi_sleep_command;
 
@@ -611,7 +621,7 @@ int main (int argc, char **argv) {
 		 * that the kernel no longer supports acpi power info, and
 		 * use hal.
 		 */
-		if (acpi_supported() &&
+		if (! force_hal && acpi_supported() &&
 		    (acpi_ac_count > 0 || acpi_batt_count > 0)) {
 			use_acpi=1;
 		}
